//-----------------------------------------------------------------------------
//Text BinaryClock

//-----------------------------------------------------------------------------
// Import - Modules used in the application
import "mod_key";
import "mod_video";
import "mod_map";
import "mod_text";
import "mod_string";
import "mod_time";
import "mod_proc";
//------------------------
//include libreria tasti Wiz e caanoo
#include "jkeys.lib"

//-----------------------------------------------------------------------------
// Constants
const version="0.6.1";        // Application Version

//-----------------------------------------------------------------------------
// Global Data
GLOBAL
int        fontId;          // ID of the font to use (currently is not loaded a custom font)

//-----------------------------------------------------------------------------
//Declare Types

//Time Type where store single time data
type TimeStruct;
	int    Hours;           //Integer value for the Hours
	string HoursString;     //String value for the Hours
	int    Minutes;         //Integer value for the Minutes
	string MinutesString;   //String value for the Hours
	int    Seconds;         //Integer value for the Seconds
	string SecondsString;   //String value for the Hours
end

//Binary structure (contains 8 Bit)
type BinaryStruct
	int Bit[7];
end

//BCD structure used to store decimal and unit values for Hour/Minute/Seconds
type BCDStruct
	BinaryStruct da;
	BinaryStruct u;
end

//-----------------------------------------------------------------------------
//Main Process
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Main Process Variables
PRIVATE
int          last_timer;    //Timer used to execute GetTime only 2 times to second
TimeStruct   currentTime;   //Current Time
BCDStruct    bcdHours;      //BCD Struct for the Hours
BCDStruct    bcdMinutes;    //BCD Struct for the Minutes
BCDStruct    bcdSeconds;    //BCD Struct for the Seconds
BinaryStruct binHours;      //BIN Struct for the Hours
BinaryStruct binMinutes;    //BIN Struct for the Minutes
BinaryStruct binSeconds;    //BIN Struct for the Seconds
//-----------------------------------------------------------------------------

BEGIN
	set_mode(320,240,16);   // Set video risolution to 320x240 pixels and 16 bits colors
    jkeys_set_default_keys(); //Setta i pulsanti
	jkeys_controller();
	//-----------------------------------------------------------------------------
	//Display information

	write(fontId,8,8,0, "Text Binary Clock v"+ version);
	write(fontId,224,2,0, "by Gigios");
	write(fontId,190,12,0, "www.OpenConsole.it");
	write(fontid,0,16,0,"________________________________________");

	// Draw BCD Text
	DrawHeaderBCD();
	DrawBCDTime(bcdHours, bcdMinutes, bcdSeconds);
	DrawFooter(currentTime, 100);

	// Draw Binary Text
	DrawHeaderBIN();
	DrawBINTime(binHours, binMinutes, binSeconds);
	DrawFooter(currentTime, 210);

	//-----------------------------------------------------------------------------
	//Set last time to force first update
	last_timer = -1000;
	//-----------------------------------------------------------------------------
	//Main Loop
	loop
		// Check timer each 500 milliseconds (+/- 2 times for seconds)
		if((Get_Timer()-last_timer)>500)
			// Get Current Time
			GetTime(currentTime);

			//-----------------------------------------------------------------------------
			// Convert Time to BCD
			GetBCDValue(bcdHours, currentTime.HoursString);
			GetBCDValue(bcdMinutes, currentTime.MinutesString);
			GetBCDValue(bcdSeconds, currentTime.SecondsString);
			
			//-----------------------------------------------------------------------------
			// Convert Time to Binary
			IntToBinaryStruct(binHours, currentTime.Hours);
			IntToBinaryStruct(binMinutes, currentTime.Minutes);
			IntToBinaryStruct(binSeconds, currentTime.Seconds);
		end
		//-----------------------------------------------------------------------------
		// Check escape key to close application
		if((key(_esc)) ||(jkeys_state[_JKEY_MENU]))
			exit();
		end
		

		frame;   //Update screen
	end
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Draw Header text for the BCD Binary Clock!
FUNCTION DrawHeaderBCD()
BEGIN
	write(fontId,10,30,0, "BCD");

	DrawHeaderTime(30);                          // Draw Hours/Minutes/Seconds Text

	// Draw "da   u" text for Hours/Minutes/Seconds
	write(fontId,74,50,0, "da");
	write(fontId,108,50,0, "u");
	write(fontId,144,50,0, "da");
	write(fontId,178,50,0, "u");
	write(fontId,214,50,0, "da");
	write(fontId,248,50,0, "u");

	// Draw vertical text (bit number)
	write(fontId,46,60,0, "8 |");
	write(fontId,46,70,0, "4 |");
	write(fontId,46,80,0, "2 |");
	write(fontId,46,90,0, "1 |");
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Get current timer and store data in the TimeStruct type
//     bcdHours (BCDStruct)   = hours to draw.
//     bcdMinutes (BCDStruct) = minutes to draw.
//     bcdSeconds (BCDStruct) = seconds to draw.
FUNCTION DrawBCDTime(BCDStruct bcdHours, BCDStruct bcdMinutes, BCDStruct bcdSeconds)
LOCAL
    int idxBit;
	int yPos;
	int yInc;
BEGIN

	yPos = 60; //Y start position
	yInc = 8;  //Y increment to write next line

	// Loop on all 4 lines of bits to show
	for(idxBit=3;idxBit>=0;idxBit--)
		// Hours
		if(idxBit < 2)
			write_int(fontId,82,yPos,0,&bcdHours.da.Bit[idxBit]);
		end
		write_int(fontId,108,yPos,0,&bcdHours.u.Bit[idxBit]);
		
		// Minutes
		if(idxBit < 3)
			write_int(fontId,152,yPos,0,&bcdMinutes.da.Bit[idxBit]);
		end
		write_int(fontId,178,yPos,0,&bcdMinutes.u.Bit[idxBit]);
		
		// Seconds
		write_int(fontId,222,yPos,0,&bcdSeconds.da.Bit[idxBit]);
		write_int(fontId,248,yPos,0,&bcdSeconds.u.Bit[idxBit]);
		
		// Increment Y position to move in the next line to draw
		yPos += yInc;
	end
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Draw Footer text and decimal time!
//     currentTime = contains current time to show as decimal value in the footer
//     yPos        = Y position where start to draw clock (100 for BCD, 210 for BIN)
FUNCTION DrawFooter(TimeStruct currentTime, int yPos)
BEGIN
	write(fontId,74,yPos,0,"===========================");

	yPos += 10;		                             // Increment Y position to move in the next line to draw

	write(fontId,10,yPos,0,"decimal:");                              // Decimal text

	write_string(fontId,90,yPos,0,&currentTime.HoursString);         // Decimal hours value
	write_string(fontId,160,yPos,0,&currentTime.MinutesString);      // Decimal minutes value
	write_string(fontId,232,yPos,0,&currentTime.SecondsString);      // Decimal seconds value
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Draw Header text for the Binary Clock!
FUNCTION DrawHeaderBIN()
BEGIN
	write(fontId,10,130,0, "BINARY");
	DrawHeaderTime(130);                         // Draw Hours/Minutes/Seconds Text

	// Draw vertical text (bit number)
	write(fontid,60,150,2, "32 |");
	write(fontid,60,160,2, "16 |");
	write(fontid,60,170,2, "8 |");
	write(fontid,60,180,2, "4 |");
	write(fontid,60,190,2, "2 |");
	write(fontid,60,200,2, "1 |");
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Draw Header Hours/Minutes/Seconds
//   yPos        = Y position where start to draw text (100 for BCD, 130 for BIN)
FUNCTION DrawHeaderTime(int yPos)
BEGIN
	// Draw Hours/Minutes/Seconds header
	write(fontId,80,yPos,0, "Hours");
	write(fontId,146,yPos,0, "Minutes");
	write(fontId,214,yPos,0, "Seconds");
	
	yPos += 10;		                             // Increment Y position to move in the next line to draw

	// Draw lines
	write(fontId,74,yPos,0, "-------");
	write(fontId,144,yPos,0, "-------");
	write(fontId,214,yPos,0, "-------");
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Get current timer and store data in the TimeStruct type
//     binHours (BinaryStruct)   = hours to draw.
//     binMinutes (BinaryStruct) = minutes to draw.
//     binSeconds (BinaryStruct) = seconds to draw.
FUNCTION DrawBINTime(BinaryStruct binHours, BinaryStruct binMinutes, BinaryStruct binSeconds)
LOCAL
    int idxBit;
	int yPos;
	int yInc;
BEGIN

	yPos = 150;	//Y start position
	yInc = 10;	//Y increment to write next line
	
	// Loop on all 5 lines of bits to show
	for(idxBit=5;idxBit>=0;idxBit--)
		// Draw bits
		write_int(fontId,93,yPos,0,&binHours.Bit[idxBit]);
		write_int(fontId,163,yPos,0,&binMinutes.Bit[idxBit]);
		write_int(fontId,235,yPos,0,&binSeconds.Bit[idxBit]);
		
		// Increment Y position to move in the next line to draw
		yPos += yInc;
    end
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Get current timer and store data in the TimeStruct type
//     t (TimeStruct) = structure where return current time
FUNCTION GetTime(TimeStruct * t)
PRIVATE
	string timeString;
BEGIN
	timeString = ftime("%H%M%S",time());         // Get Time in string format ("HHMMSS")

	t.HoursString = substr(timeString,0,2);      // Extract only the hours from the time string
	t.Hours = ITOA(t.HoursString);               // Convert string to integer

	t.MinutesString = substr(timeString,2,2);    // Extract only the minutes from the time string
	t.Minutes = ITOA(t.MinutesString);           // Convert string to integer

	t.SecondsString = substr(timeString,4,2);    // Extract only the seconds from the time string
	t.Seconds = ITOA(t.SecondsString);           // Convert string to integer
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Convert string value (two chars) into bcd structure
//     bcd (BCDStruct)         = structure where return data
//     valueToConvert (string) = value to convert (two chars)
FUNCTION GetBCDValue(BCDStruct * bcd, string valueToConvert)
PRIVATE
	int da;
	int u;
BEGIN
	da = ATOI(substr(valueToConvert, 0, 1));     // Decimal value
	u = ATOI(substr(valueToConvert, 1, 1));      // Unit value
	
	IntToBinaryStruct(bcd.da, da);               // Extract bits
	IntToBinaryStruct(bcd.u, u);                 // Extract bits
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Convert integer value into binary structure
//     bin (BinaryStruct)   = structure where return data
//     valueToConvert (int) = value to convert
FUNCTION IntToBinaryStruct(BinaryStruct * bin, int valueToConvert)
PRIVATE
	int n;
	int idxBit;
BEGIN
	n = valueToConvert;                          //Store locally the value to convert
	
	// Reset all 8 bits in the structure
	for(idxBit=0;idxBit<=7;idxBit+=1)
        bin.bit[idxBit] = 0;
    end
	
	// Get all bits in the integer value and store it in the structure
	idxBit = 0;
	while(n >= 1)
		bin.bit[idxBit] = (n mod 2);               //Check the modulus of the division with 2 
		n = n / 2;
		//n = n >> 1;                              //Shift right the value to execute division for 2
		idxBit++;
	end
END
//-----------------------------------------------------------------------------
